package com.xynet.marketing.utils.redis.utils;

import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.data.redis.core.HashOperations;
import org.springframework.data.redis.core.ListOperations;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.data.redis.support.atomic.RedisAtomicLong;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.concurrent.TimeUnit;

@Service
public class RedisServiceImpl<V> implements RedisService<V> {

    private RedisTemplate<String, V> redisTemplate;

    public RedisServiceImpl(@Qualifier("redisTemplateSerializer") RedisTemplate redisTemplate) {
        this.redisTemplate = redisTemplate;
    }

    private ValueOperations<String, V> getValueOperations() {
        return redisTemplate.opsForValue();
    }

    private ListOperations<String, V> getListOperations() {
        return redisTemplate.opsForList();
    }

    private HashOperations<String, String, V> getHashOperations() {
        return redisTemplate.opsForHash();
    }

    @Override
    public void timeout(String key, long timeout) {
        redisTemplate.expire(key, timeout, TimeUnit.SECONDS);
    }

    @Override
    public void timeout(String key, long timeout, TimeUnit timeUnit) {
        redisTemplate.expire(key, timeout, timeUnit);
    }

    @Override
    public long getTimeout(String key) {
        return redisTemplate.getExpire(key);
    }

    @Override
    public long getTimeout(String key, TimeUnit timeUnit) {
        return redisTemplate.getExpire(key, timeUnit);
    }

    @Override
    public void set(String key, V value) {
        getValueOperations().set(key, value);
    }

    @Override
    public void set(String key, V value, long timeout) {
        set(key, value);
        redisTemplate.expire(key, timeout, TimeUnit.SECONDS);
    }

    @Override
    public void set(Map<String, V> map) {
        getValueOperations().multiSet(map);
    }

    @Override
    public void set(Map<String, V> map, long timeout) {
        set(map);
        map.forEach((key, value) -> redisTemplate.expire(key, timeout, TimeUnit.SECONDS));
    }

    @Override
    public boolean exists(String key) {
        return redisTemplate.hasKey(key);
    }

    @Override
    public V get(String key) {
        return getValueOperations().get(key);
    }

    @Override
    public Map<String, V> get(List<String> keys) {
        Map<String, V> map = new HashMap<>(keys.size());
        keys.forEach(key -> {
            V v = get(key);
            if (v != null) {
                map.put(key, v);
            }
        });
        return map;
    }

    @Override
    public Map<String, V> like(String prex) {
        Set<String> keys = redisTemplate.keys("*" + prex + "*");
        Map<String, V> map = new HashMap<>(keys.size());
        if (CollectionUtils.isEmpty(keys)) {
            return map;
        }
        keys.forEach(key -> {
            V v = get(key);
            map.put(key, v);
        });
        return map;
    }

    @Override
    public V remove(String key) {
        V v = get(key);
        redisTemplate.delete(key);
        return v;
    }

    @Override
    public Map<String, V> remove(List<String> keys) {
        Map<String, V> map = get(keys);
        redisTemplate.delete(keys);
        return map;
    }

    @Override
    public Map<String, V> removeLike(String prex) {
        Set<String> keys = redisTemplate.keys("*" + prex + "*");
        Map<String, V> map = new HashMap<>(keys.size());
        if (CollectionUtils.isEmpty(keys)) {
            return map;
        }
        keys.forEach(key -> {
            V v = remove(key);
            map.put(key, v);
        });
        return map;
    }

    @Override
    public void setList(String key, V v) {
        getListOperations().rightPush(key, v);
    }

    @Override
    public void setList(String key, List<V> list) {
        getListOperations().rightPushAll(key, list);
    }

    @Override
    public V getFirstListAndRemove(String key) {
        return getListOperations().leftPop(key);
    }

    @Override
    public V getLastListAndRemove(String key) {
        return getListOperations().rightPop(key);
    }

    @Override
    public List<V> getList(String key) {
        ListOperations<String, V> listOperations = getListOperations();
        List<V> list = listOperations.range(key, 0, listOperations.size(key));
        return list;
    }

    @Override
    public List<V> getLikeList(String key) {
        Set<String> keys = redisTemplate.keys("*" + key + "*");
        List<V> list = new ArrayList<>();
        keys.forEach(s -> list.addAll(getList(s)));
        return list;
    }

    @Override
    public int getListSize(String key) {
        long size = getListOperations().size(key);
        return (int) size;
    }

    @Override
    public void updateList(String key, int index, V value) {
        getListOperations().set(key, index, value);
    }

    @Override
    public V removeList(String key, int index) {
        ListOperations<String, V> listOperations = getListOperations();
        V v = listOperations.index(key, index);
        listOperations.remove(key, 0, v);
        return v;
    }

    @Override
    public boolean removeList(String key, V v) {
        Long remove = getListOperations().remove(key, 0, v);
        return remove > 0 ? true : false;
    }

    @Override
    public void removeList(String key) {
        redisTemplate.delete(key);
    }

    @Override
    public List<V> removeLikeList(String key) {
        Set<String> keys = redisTemplate.keys("*" + key + "*");
        List<V> list = new ArrayList<>();
        keys.forEach(s -> {
            list.addAll(getList(s));
            redisTemplate.delete(s);
        });
        return list;
    }

    @Override
    public void setMap(String key, String mk, V mv) {
        getHashOperations().put(key, mk, mv);
    }

    @Override
    public void setMap(String key, Map<String, V> map) {
        getHashOperations().putAll(key, map);
    }

    @Override
    public Map<String, V> getMap(String key) {
        return getHashOperations().entries(key);
    }

    @Override
    public V getMap(String key, String mk) {
        return getHashOperations().get(key, mk);
    }

    @Override
    public Map<String, Map<String, V>> likeMap(String prex) {
        Set<String> keys = getHashOperations().getOperations().keys("*" + prex + "*");
        Map<String, Map<String, V>> maps = new HashMap<>(keys.size());
        if (CollectionUtils.isEmpty(keys)) {
            return maps;
        }
        keys.forEach(key -> {
            Map<String, V> map = getMap(key);
            maps.put(key, map);
        });
        return maps;
    }

    @Override
    public int getMapSize(String key) {
        return Integer.valueOf(getHashOperations().size(key).toString());
    }

    @Override
    public void removeMap(String key) {
        redisTemplate.delete(key);
    }

    @Override
    public void removeMap(String key, String mk) {
        getHashOperations().delete(key, mk);
    }

    @Override
    public void removeLikeMap(String prex) {
        Set<String> keys = getHashOperations().getOperations().keys("*" + prex + "*");
        keys.forEach(key -> removeMap(key));
    }

    @Override
    public long atomicIncrease(String key) {
        return new RedisAtomicLong(key, redisTemplate.getConnectionFactory()).getAndIncrement();
    }

    @Override
    public long atomicReduce(String key) {
        return new RedisAtomicLong(key, redisTemplate.getConnectionFactory()).getAndDecrement();
    }

    @Override
    public long setAtomic(String key, long value) {
        return new RedisAtomicLong(key, redisTemplate.getConnectionFactory()).getAndSet(value);
    }

    @Override
    public long getAtomic(String key) {
        return new RedisAtomicLong(key, redisTemplate.getConnectionFactory()).get();
    }

    @Override
    public void atomicEmpty(String key) {
        remove(key);
    }

}
